@@ -19,6 +19,8 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
+ * (C) Copyright 2009 - 2011 Red Hat, Inc.
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +49,7 @@
 #include <dbus/dbus-glib.h>
 
 #include "nma-marshal.h"
-#include "mobile-wizard.h"
+#include "nm-mobile-wizard.h"
 
 #define DBUS_TYPE_G_MAP_OF_VARIANT (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE))
 
@@ -94,7 +96,7 @@ typedef struct {
 	char *rfcomm_iface;
 	guint dun_timeout_id;
 
-	MobileWizard *wizard;
+	NMAMobileWizard *wizard;
 	GtkWindowGroup *window_group;
 } PluginInfo;
 
@@ -314,6 +316,11 @@ dun_cleanup (PluginInfo *info, const char *message, gboolean uncheck)
 	g_slist_free (info->modem_proxies);
 	info->modem_proxies = NULL;
 
+	if (info->mm_proxy) {
+		g_object_unref (info->mm_proxy);
+		info->mm_proxy = NULL;
+	}
+
 	if (info->dun_proxy) {
 		if (info->rfcomm_iface) {
 			dbus_g_proxy_call_no_reply (info->dun_proxy, "Disconnect",
@@ -331,11 +338,6 @@ dun_cleanup (PluginInfo *info, const char *message, gboolean uncheck)
 	g_free (info->rfcomm_iface);
 	info->rfcomm_iface = NULL;
 
-	if (info->bus) {
-		dbus_g_connection_unref (info->bus);
-		info->bus = NULL;
-	}
-
 	if (info->dun_timeout_id) {
 		g_source_remove (info->dun_timeout_id);
 		info->dun_timeout_id = 0;
@@ -347,7 +349,7 @@ dun_cleanup (PluginInfo *info, const char *message, gboolean uncheck)
 	}
 
 	if (info->wizard) {
-		mobile_wizard_destroy (info->wizard);
+		nma_mobile_wizard_destroy (info->wizard);
 		info->wizard = NULL;
 	}
 
@@ -377,7 +379,7 @@ dun_error (PluginInfo *info, const char *func, GError *error, const char *fallba
 }
 
 static NMConnection *
-dun_new_cdma (MobileWizardAccessMethod *method)
+dun_new_cdma (NMAMobileWizardAccessMethod *method)
 {
 	NMConnection *connection;
 	NMSetting *setting;
@@ -425,7 +427,7 @@ dun_new_cdma (MobileWizardAccessMethod *method)
 }
 
 static NMConnection *
-dun_new_gsm (MobileWizardAccessMethod *method)
+dun_new_gsm (NMAMobileWizardAccessMethod *method)
 {
 	NMConnection *connection;
 	NMSetting *setting;
@@ -493,9 +495,9 @@ dun_add_cb (NMRemoteSettings *settings,
 }
 
 static void
-wizard_done_cb (MobileWizard *self,
+wizard_done_cb (NMAMobileWizard *self,
                 gboolean canceled,
-                MobileWizardAccessMethod *method,
+                NMAMobileWizardAccessMethod *method,
                 gpointer user_data)
 {
 	PluginInfo *info = user_data;
@@ -519,7 +521,7 @@ wizard_done_cb (MobileWizard *self,
 		return;
 	}
 
-	mobile_wizard_destroy (info->wizard);
+	nma_mobile_wizard_destroy (info->wizard);
 	info->wizard = NULL;
 
 	g_assert (connection);
@@ -628,13 +630,13 @@ modem_get_all_cb (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 		}
 
 		/* Start the mobile wizard */
-		info->wizard = mobile_wizard_new (parent ? GTK_WINDOW (parent) : NULL,
-		                                  info->window_group,
-		                                  devtype,
-		                                  FALSE,
-		                                  wizard_done_cb,
-		                                  info);
-		mobile_wizard_present (info->wizard);
+		info->wizard = nma_mobile_wizard_new (parent ? GTK_WINDOW (parent) : NULL,
+											  info->window_group,
+											  devtype,
+											  FALSE,
+											  wizard_done_cb,
+											  info);
+		nma_mobile_wizard_present (info->wizard);
 	}
 
 out:
@@ -942,7 +944,14 @@ plugin_info_destroy (gpointer data)
 		gtk_spinner_stop (GTK_SPINNER (info->spinner));
 	g_object_unref (info->settings);
 	g_object_unref (info->btmodel);
+
+	g_signal_handlers_disconnect_matched (info->btclient,
+	                                      G_SIGNAL_MATCH_DATA, 0, 0, NULL,
+	                                      NULL, info);
 	g_object_unref (info->btclient);
+
+	if (info->bus)
+		dbus_g_connection_unref (info->bus);
 	memset (info, 0, sizeof (PluginInfo));
 	g_free (info);
 }
@@ -1077,7 +1086,11 @@ get_config_widgets (const char *bdaddr, const char **uuids)
 			  G_CALLBACK (default_adapter_powered_changed), info);
 
 	/* UI setup */
+#if GTK_CHECK_VERSION (3,1,6)
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	g_object_set_data_full (G_OBJECT (vbox), "info", info, plugin_info_destroy);
 
 	if (pan) {
@@ -1098,11 +1111,19 @@ get_config_widgets (const char *bdaddr, const char **uuids)
 		gtk_box_pack_start (GTK_BOX (vbox), info->dun_button, FALSE, TRUE, 6);
 	}
 
+#if GTK_CHECK_VERSION (3,1,6)
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
 	hbox = gtk_hbox_new (FALSE, 6);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 6);
 
 	/* Spinner's hbox */
+#if GTK_CHECK_VERSION (3,1,6)
+	info->hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
 	info->hbox = gtk_hbox_new (FALSE, 6);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), info->hbox, FALSE, FALSE, 0);
 
 	info->label = gtk_label_new ("");
